#include "GDGeometry.h"


#ifdef GD_USE_GPU
#include <opencv2/gpu/gpu.hpp>
static bool s_bGPU = true;
#else
static bool s_bGPU = false;
#endif

// Straight line ax+by+c=0
double distancePointLine(double x, double y, double a, double b, double c)
{
  return abs(a*x+b*y+c)/sqrt(a*a+b*b);
}
// Straight line y=ax+b
double distancePointLine(double x, double y, double a, double b)
{
  //return distancePointDroite(x1, y1, a, -1, b);
  return abs(a*x-y+b)/sqrt(a*a+1);
}

// x, y are point on the plane where is a line y=ax+b 
// output x_rel, y_rel are points relative to the coordinate system defined with :
// -> x_on_line, y_on_line are the origin
// -> X axis is the line y=ax+b
void xy_to_xy_line_relative(double x, double y, 
                            double a, double x_on_line, double y_on_line, 
                            double& x_rel, double& y_rel)
{
  double denum = sqrt(a*a+1);
  //y_rel = abs(a*x-y+b)/denum;
  double dx = 1/denum;
  double dy = a/denum;
  x_rel = (y-y_on_line)*dy + (x-x_on_line)*dx;
  y_rel = (x-x_on_line)*dy - (y-y_on_line)*dx;
}

CalibrationGeo::CalibrationGeo()
{
}

CalibrationGeo::~CalibrationGeo()
{
}

void CalibrationGeo::Calibrate(Point2f* ptPtImage, Point2f* ptPtWorld, int nCount)
{   
  //Mat(3, 3, CV_32FC1, m_f32MxPerspective);
  if(nCount == 3)
    m_mx1DPerspective = getAffineTransform(ptPtImage, ptPtWorld);
  if(nCount == 4)
    m_mx1DPerspective = getPerspectiveTransform(ptPtImage, ptPtWorld);
}

void CalibrationGeo::InitLUT(int nSzX, int nSzY)
{  
  Mat mx2DLUT(nSzX,nSzY,CV_64FC2);

  Mat& M1D = m_mx1DPerspective;
  double tdst;
  for(int ysrc=0; ysrc<nSzY; ysrc++){
    for(int xsrc=0; xsrc<nSzX; xsrc++){
      tdst =  xsrc * M1D.at<double>(6) + ysrc *M1D.at<double>(7) + M1D.at<double>(8);
      mx2DLUT.at<Vec2d>(xsrc, ysrc)[0] = (xsrc * M1D.at<double>(0) + ysrc * M1D.at<double>(1) + M1D.at<double>(2)) / tdst;
      mx2DLUT.at<Vec2d>(xsrc, ysrc)[1] = (xsrc * M1D.at<double>(3) + ysrc * M1D.at<double>(4) + M1D.at<double>(5)) / tdst;
    }
  }
  m_mx2DLUT = mx2DLUT; 
}

// for boost
// #include <boost/timer/timer.hpp>
// #include <boost/format.hpp>

void CalibrationGeo::Transform(const Mat imgSrc, Mat imgDst)
{
  //INTER_NEAREST, WARP_INVERSE_MAP
  if(s_bGPU)
  {
#ifdef GD_USE_GPU
    gpu::GpuMat src_gpu, dst_gpu;
    src_gpu.upload(imgSrc);
    src_gpu.upload(imgDst);
	  //boost::timer::cpu_timer timer;
	  //timer.start();
    gpu::warpPerspective(src_gpu, dst_gpu, m_mx1DPerspective, imgDst.size(), cv::INTER_LINEAR);
#endif
  }
  else
  {    
    if(m_mx1DPerspective.rows == 3)
      warpPerspective(imgSrc, imgDst, m_mx1DPerspective, imgDst.size());
    else
      warpAffine(imgSrc, imgDst, m_mx1DPerspective, imgDst.size());
  }
}

//The explicit instantiation part
//template class CalibrationGeo<float>; 