#include <QtWidgets>

#include "QColorimeterDlg.h"
#include "QOpenCV.h"

QColorimeterDlg::QColorimeterDlg()
{
  m_pQCVImage= NULL;

   QVBoxLayout *qMainLayout = new QVBoxLayout;

   QGroupBox* pQGroupBoxForm = new QGroupBox(tr("Mean Lab"));
   QFormLayout *pQFormLayout = new QFormLayout;
   m_pSpin_L = new QSpinBox; m_pSpin_L->setMaximum(100);
   m_pSpin_a = new QSpinBox; m_pSpin_a->setMinimum(-300); m_pSpin_a->setMaximum(299);
   m_pSpin_b = new QSpinBox; m_pSpin_b->setMinimum(-300); m_pSpin_b->setMaximum(299);
   QPushButton* pqPushButtonSetLab = new QPushButton(tr("Set Mean Lab Value"));
   pQFormLayout->addRow(new QLabel(tr("L:")), m_pSpin_L);
   pQFormLayout->addRow(new QLabel(tr("a:")), m_pSpin_a);
   pQFormLayout->addRow(new QLabel(tr("b:")), m_pSpin_b);
   pQFormLayout->addRow(pqPushButtonSetLab);

   pQGroupBoxForm->setLayout(pQFormLayout);
   qMainLayout->addWidget(pQGroupBoxForm);

   QVBoxLayout* pqLayoutButtons = new QVBoxLayout;
   //QPushButton* pqPushButtonSetLab = new QPushButton(tr("Set Mean Lab Value"));
   QPushButton* pqPushButtonCreateLab = new QPushButton(tr("Create Delta Lab images"));
   
   //pqLayoutButtons->addWidget(pqPushButtonSetLab);
   pqLayoutButtons->addWidget(pqPushButtonCreateLab);
   qMainLayout->addLayout(pqLayoutButtons);

   setLayout(qMainLayout);

   connect(pqPushButtonSetLab, SIGNAL(clicked()), this, SLOT(SetMeanLab()));//released
}

void QColorimeterDlg::SetMeanLab()
{
  SetLab(m_pSpin_L->value() * 255 / 100, 
         (m_pSpin_a->value() + 300) * 255 / 600, 
         (m_pSpin_b->value() + 300) * 255 / 600);
}

void QColorimeterDlg::SetLab(int nL, int nA, int nB)
{
  if(!m_pQCVImage)
    return;

  cv::Mat mxSrc;
  cv::Mat mxDst;

  cv::Point* pROI = m_pQCVImage->getROI(0);
  int nOffX = pROI[0].x;
  int nOffY = pROI[0].y;
  int nSzX = pROI[3].x-pROI[0].x;
  int nSzY = pROI[1].y-pROI[0].y;
  
  if(m_nDstCnt == 0)
  { 
    mxSrc = m_pQCVImage->getImg();
    mxDst = cv::Mat(mxSrc.rows, mxSrc.cols, mxSrc.type());
    addImg(mxSrc, true);
    addImg(mxDst, false);
    m_pQCVImage->addImg(getImg(false));
  }
  else
  {
    mxSrc = getImg(true);
    mxDst = getImg(false);
  }

  cv::Mat mxLab_U8;
  
  cvtColor(mxSrc, mxLab_U8, CV_LBGR2Lab);
  
  cv::Mat3b roiMat = mxLab_U8(cv::Rect(nOffX, nOffY, nSzX, nSzY));

  cv::Scalar mean = cv::mean(roiMat);
  cv::Vec3f destmean;

  destmean.val[0] = nL / mean.val[0];
  destmean.val[1] = nA / mean.val[1];
  destmean.val[2] = nB / mean.val[2];
    
  for(int r=0; r<nSzY; r++)
  {
    for(int c=0; c<nSzX; c++)
    {
      cv::Vec3b& v = mxLab_U8.at<cv::Vec3b>(nOffY+r, nOffX+c);
      v.val[0] = (uchar) v.val[0] * destmean.val[0];
      v.val[1] = (uchar) v.val[1] * destmean.val[1];
      v.val[2] = (uchar) v.val[2] * destmean.val[2];
    }
  }
  
  cvtColor(mxLab_U8, mxDst, CV_Lab2LBGR);

  m_pQCVImage->update();
}

void QColorimeterDlg::addQCVImage(QCVImage* pQCVImage)
{
  m_pQCVImage = pQCVImage;
}