#include <QtGui>
#if QT_VERSION > 0x050000
#include <QtWidgets>
#include <QtPrintSupport>
#endif
#include "QImageScrollArea.h"
#include "QOpenCV.h"

QImageScrollArea::QImageScrollArea()
{
  m_fitToWindowAct = NULL;

  m_QCVImage.setBackgroundRole(QPalette::Base);
  m_QCVImage.setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  m_QCVImage.setScaledContents(true);

  setBackgroundRole(QPalette::Dark);
  setWidget(&m_QCVImage);

  createActions();

  m_QCVImage.load(QString("C:/Images-Acquisitions/20130304_kleurmetingen/DSC_1245.tif"));

  update();
}

void QImageScrollArea::dropEvent (QDropEvent* pEvent)
{
}

void QImageScrollArea::mousePressEvent(QMouseEvent* pEvent)
{
  //emit signalMousePress(m_QCVImage.GetScroll().x() + pEvent->x(), m_QCVImage.GetScroll().y() + pEvent->y());
}

void QImageScrollArea::mouseReleaseEvent(QMouseEvent* pEvent)
{
}

void QImageScrollArea::wheelEvent(QWheelEvent* pEvent)
{
   double dDegrees = (double)(pEvent->delta() / 8);
   
   scaleImage(pow(1.25, dDegrees / 15.0));
}

void QImageScrollArea::keyPressEvent(QKeyEvent* pEvent)
{
  switch(pEvent->key())
  {
  case 87://w
    m_QCVImage.Warp();
    m_QCVImage.ShadingCorrection();
    break;
  }
}

void QImageScrollArea::slotSliderH_ValueChanged ( int nMin)
{
  //m_QCVImage.GetScroll().setX(nMin);
};

void QImageScrollArea::slotSliderV_ValueChanged ( int nMin)
{
  //m_QCVImage.GetScroll().setY(nMin);
};

void QImageScrollArea::newSlot()
{
}

void QImageScrollArea::openSlot()
{
  QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), QDir::currentPath());

  if (!fileName.isEmpty())
  {
    if (!m_QCVImage.load(fileName)) 
    {
      QMessageBox::information(this, tr("QBaseImage"), tr("Cannot load Error %1.").arg(fileName));
      return;
    }
    else
    {
      update();
    }
  }
}

void QImageScrollArea::saveSlot()
{
  QString fileName = QFileDialog::getSaveFileName(this,
    tr("Choose a file name"), ".",
    tr("img (*.tif  *.png*.bmp *.jpg *.jp2 *.pbm *.sr)"));
  if (fileName.isEmpty())
    return;
  QFile file(fileName);
  if (!m_QCVImage.save(fileName)) {
    QMessageBox::warning(this, tr("Calibration App"),
      tr("Cannot write file %1:\n%2.")
      .arg(fileName)
      .arg(file.errorString()));
    return;
  }
  //statusBar()->showMessage(tr("Saved '%1'").arg(fileName), 2000);
}

void QImageScrollArea::print()

{
  Q_ASSERT(m_QCVImage.pixmap());
#ifndef QT_NO_PRINTER
  QPrintDialog dialog(&m_Printer, this);

  if (dialog.exec()) {
    QPainter painter(&m_Printer);
    QRect rect = painter.viewport();
    QSize size = m_QCVImage.pixmap()->size();
    size.scale(rect.size(), Qt::KeepAspectRatio);
    painter.setViewport(rect.x(), rect.y(), size.width(), size.height());
    painter.setWindow(m_QCVImage.pixmap()->rect());
    painter.drawPixmap(0, 0, *m_QCVImage.pixmap());
  }
#endif
}

void QImageScrollArea::zoomIn()
{
  scaleImage(1.25);
}

void QImageScrollArea::zoomOut()
{
  scaleImage(0.8);
}

void QImageScrollArea::normalSize()
{
  scaleImage(1.0);
}

void QImageScrollArea::warp()
{
  m_QCVImage.Warp();
}

void QImageScrollArea::shading()
{
  m_QCVImage.ShadingCorrection();
}

void QImageScrollArea::fitToWindow()
{
  bool fitToWindow = m_fitToWindowAct->isChecked();

  setWidgetResizable(fitToWindow);
  if (!fitToWindow) 
  {
    normalSize();
  }
  updateActions();
}

void QImageScrollArea::createActions()
{
  m_printAct = new QAction(QIcon(":/images/print.png"), tr("&Print..."), this);
  m_printAct->setShortcuts(QKeySequence::Print);
  m_printAct->setStatusTip(tr("Print the current image"));
  m_printAct->setEnabled(false);
  connect(m_printAct, SIGNAL(triggered()), this, SLOT(print()));

  m_zoomInAct = new QAction(tr("Zoom &In (25%)"), this);
  m_zoomInAct->setShortcut(tr("Ctrl++"));
  m_zoomInAct->setEnabled(false);
  connect(m_zoomInAct, SIGNAL(triggered()), this, SLOT(zoomIn()));

  m_zoomOutAct = new QAction(tr("Zoom &Out (25%)"), this);
  m_zoomOutAct->setShortcut(tr("Ctrl+-"));
  m_zoomOutAct->setEnabled(false);
  connect(m_zoomOutAct, SIGNAL(triggered()), this, SLOT(zoomOut()));

  m_normalSizeAct = new QAction(tr("&Normal Size"), this);
  m_normalSizeAct->setShortcut(tr("Ctrl+S"));
  m_normalSizeAct->setEnabled(false);
  connect(m_normalSizeAct, SIGNAL(triggered()), this, SLOT(normalSize()));

  m_fitToWindowAct = new QAction(tr("&Fit to Window"), this);
  m_fitToWindowAct->setEnabled(false);
  m_fitToWindowAct->setCheckable(true);
  m_fitToWindowAct->setShortcut(tr("Ctrl+F"));
  connect(m_fitToWindowAct, SIGNAL(triggered()), this, SLOT(fitToWindow()));

  connect(this->horizontalScrollBar(), SIGNAL(valueChanged (int)), this, SLOT(slotSliderH_ValueChanged(int)));
  connect(this->verticalScrollBar(),  SIGNAL(valueChanged (int)), this, SLOT(slotSliderV_ValueChanged(int)));
}

QMenu* QImageScrollArea::createMenus()
{
  m_pMenu = new QMenu(tr("&Image"), this);
  m_pMenu->addAction(m_zoomInAct);
  m_pMenu->addAction(m_zoomOutAct);
  m_pMenu->addAction(m_normalSizeAct);
  m_pMenu->addAction(m_fitToWindowAct);
  m_pMenu->addAction(m_printAct);

  return m_pMenu;
}

void QImageScrollArea::updateActions()
{
  if(m_fitToWindowAct == NULL)
    return;

  if (!m_fitToWindowAct->isChecked())
    m_QCVImage.adjustSize();

  m_printAct->setEnabled(true);
  m_fitToWindowAct->setEnabled(true);

  m_zoomInAct->setEnabled(!m_fitToWindowAct->isChecked());
  m_zoomOutAct->setEnabled(!m_fitToWindowAct->isChecked());
  m_normalSizeAct->setEnabled(!m_fitToWindowAct->isChecked());
}

void QImageScrollArea::update()
{
  updateActions();

  m_QCVImage.update();
}

void QImageScrollArea::scaleImage(double factor)
{
  if(m_QCVImage.isEmpty())
    return;

  m_QCVImage.zoom(factor);

  if(factor != 1.0)
  {
    adjustScrollBar(horizontalScrollBar(), factor);
    adjustScrollBar(verticalScrollBar(), factor);
  }
}

void QImageScrollArea::adjustScrollBar(QScrollBar *scrollBar, double factor)
{
  scrollBar->setValue(int(factor * scrollBar->value() + ((factor - 1) * scrollBar->pageStep()/2)));
}



