#include "QOpenCV.h"
#include "GDGeometry.h"

//opencv_core230d.lib; opencv_contrib230d.lib; opencv_imgproc230d.lib; opencv_highgui230d.lib
#ifdef _DEBUG
#pragma comment(lib, "C:\\Dev\\lib\\x86\\opencv_core220d.lib")
#pragma comment(lib, "C:\\Dev\\lib\\x86\\opencv_imgproc220d.lib")
#pragma comment(lib, "C:\\Dev\\lib\\x86\\opencv_highgui220d.lib")
#else
#pragma comment(lib, "opencv_core220.lib")
#pragma comment(lib, "opencv_imgproc220.lib")
#pragma comment(lib, "opencv_highgui220.lib")
#endif

CVProcessing::CVProcessing()
{
  m_nSrcCnt = m_nDstCnt = 0;
}

bool CVProcessing::addImg(cv::Mat& newImg, bool bSrc)
{
  if(!newImg.empty())
  {
    if(bSrc)
    {
      m_MatSrc[m_nSrcCnt++] = newImg;
    }
    else
    {
      m_MatDst[m_nDstCnt++] = newImg;
    }
    return true;
  }
  else
  {
    return false;
  }
}

cv::Mat& CVProcessing::getImg(bool bSrc, int nIdx, bool bLastBased)
{
  if(bSrc)
  {
    if(bLastBased)
      nIdx = m_nSrcCnt-1+nIdx;
    else
      nIdx = (nIdx==-1) ? m_nSrcCnt-1 : nIdx%MAX_IMG_CNT;
  
    nIdx = max(m_nSrcCnt-1, nIdx);

    return m_MatSrc[nIdx];
  }
  else
  {
    if(bLastBased)
      nIdx = m_nDstCnt-1+nIdx;
    else
      nIdx = (nIdx==-1) ? m_nDstCnt-1 : nIdx%MAX_IMG_CNT;
  
    nIdx = max(m_nDstCnt-1, nIdx);

    return m_MatDst[nIdx];
  }
}

void CVProcessing::addROI(cv::Point* ptROI, int nPtCnt)
{
}

void CVProcessing::addROI(cv::Rect rcROI)
{
}

cv::Point* CVProcessing::getROI(int nID)
{
  if(nID == 0)
    return m_ptROISrc;
  else
    return m_ptROIDst;
}

//QImage(m_MatSrc.data, m_MatSrc.cols, m_MatSrc.rows, m_MatSrc.step, QImage::Format_RGB888);  
//QPixmap qpix = QPixmap::fromImage(qimg.rgbSwapped()); //BGR to RGB
QImage Mat8UC1_2QImage(const cv::Mat& src)
{       
  QImage dest;
  //ColorTable for Binary Images
  QVector<QRgb> colorTable;
  for (int i = 0; i < 256; i++)
    colorTable.push_back(qRgb(i, i, i));

  dest = QImage((const unsigned char*)src.data, src.cols, src.rows, QImage::Format_Indexed8);
  dest.setColorTable(colorTable);

  return dest;
}

QImage Mat8UC3_2QImage(const cv::Mat& src)
{       
  QImage dest(src.cols, src.rows, QImage::Format_ARGB32);
  for (int y = 0; y < src.rows; ++y)
  {
    const cv::Vec3b *srcrow = (const cv::Vec3b*)(src.data+src.step*y);
    QRgb *destrow = (QRgb*)dest.scanLine(y);
    for (int x = 0; x < src.cols; ++x)
    {
      destrow[x] = qRgba(srcrow[x][2], srcrow[x][1], srcrow[x][0], 255);
    } 
  }
  return dest;
}

QImage Mat64FC1_2QImage(const cv::Mat& src)
{
  double scale = 255.0;
  QImage dest(src.cols, src.rows, QImage::Format_ARGB32);
  for (int y = 0; y < src.rows; ++y)
  {
    const double *srcrow = (const double*)(src.data+src.step*y);
    QRgb *destrow = (QRgb*)dest.scanLine(y);
    for (int x = 0; x < src.cols; ++x) 
    {
      unsigned int color = srcrow[x] * scale;
      destrow[x] = qRgba(color, color, color, 255);
    }
  }
  return dest;
}

QCVImage::QCVImage()
{
  m_f64Zoom = 1.0;
  m_nHandleMove = m_nHandleStart = ROI_HANDLE_OUT; 
  /*
  m_ptScroll.setX(0);
  m_ptScroll.setY(0);
  */
  setMouseTracking(true);
}

QCVImage::QCVImage(QString& strPath)
{
  load(strPath);
}

QCVImage::~QCVImage()
{
}

/*
QPoint& QCVImage::GetScroll()
{
  return m_ptScroll;
}
*/
bool QCVImage::isEmpty()
{
  return getImg().empty();
}

bool QCVImage::load(QString& strPath)
{ 
  if(addImg(cv::imread(strPath.toUtf8().constData(), CV_LOAD_IMAGE_UNCHANGED)))//CV_LOAD_IMAGE_ANYDEPTH
  {
    //cvConvertImage(getImg(), getImg(), CV_CVTIMG_SWAP_RB);
    m_ptROISrc[0].x = m_ptROISrc[1].x = getImg().cols / 4;
    m_ptROISrc[2].x = m_ptROISrc[3].x = getImg().cols * 3 / 4;
    m_ptROISrc[0].y = m_ptROISrc[3].y = getImg().rows / 4;
    m_ptROISrc[1].y = m_ptROISrc[2].y = getImg().rows * 3 / 4;

    update();
    return true;
  }
  else
    return false;
}  

bool QCVImage::save(QString& strPath)
{
  return cv::imwrite(strPath.toUtf8().constData(), getImg());
}

void QCVImage::mouseMoveEvent(QMouseEvent* pEvent)
{
  mouseMouve(pEvent->x(), 
             pEvent->y(), 
             pEvent->buttons());
}

/*
m_LPixmap = new QPixmap(32,32);
m_LPixmap->fill(Qt::transparent); // Otherwise you get a black background :(
QPainter painter(m_LPixmap);
QColor red(255,0,0,128);

painter.setPen(Qt::NoPen);        // Otherwise you get an thin black border
painter.setBrush(red);

painter.drawEllipse(0,0,32,32);
m_Cursor = QCursor(*m_LPixmap);
setCursor(m_Cursor);
*/
/*
        Format_Invalid,
        Format_Mono,
CV_8UC1 Format_Indexed8, 8-bit indexes into a colormap
        Format_RGB32, 0xFFRRGGBB
        Format_ARGB32,  0xAARRGGBB
        Format_ARGB32_Premultiplied,  0xAARRGGBB
CV_8UC3 Format_RGB888
*/
#define ROI_HANDLER_SZ 10
#define ROI_HANDLER_HF 5
void QCVImage::paintEvent(QPaintEvent* pQPaintEvent)
{
  QLabel::paintEvent(pQPaintEvent);
  
  QPainter painter(this);
/*
  IplImage iplImg = getImg();
  QImage qimg(getImg().ptr(), getImg().cols, getImg().rows, iplImg.widthStep, (getImg().type() == CV_8UC3) ? Format_RGB888 : Format_Indexed8);
  painter.drawPixmap()
  */
  //QPen penROI(Qt::green, 1, Qt::SolidLine);
  QPen penROI(QBrush(QColor(169, 1, 219)), 1, Qt::SolidLine);//#B4045F #8A084B #A901DB

  painter.setPen(penROI);

  for(int i=0; i<4; i++)
  {
    painter.drawRect((m_ptROISrc[i].x)*m_f64Zoom - ROI_HANDLER_HF, 
                     (m_ptROISrc[i].y)*m_f64Zoom - ROI_HANDLER_HF,
                     ROI_HANDLER_SZ, 
                     ROI_HANDLER_SZ);
    painter.drawLine(m_ptROISrc[i].x*m_f64Zoom,
                     m_ptROISrc[i].y*m_f64Zoom, 
                     m_ptROISrc[(i+1)%4].x*m_f64Zoom, 
                     m_ptROISrc[(i+1)%4].y*m_f64Zoom);
  }
}

void QCVImage::update()
{
  switch(getImg().type())
  {
  case CV_8UC1:
    m_QPixmap = QPixmap::fromImage(Mat8UC1_2QImage(getImg()));
    break;
  case CV_8UC3:
    m_QPixmap = QPixmap::fromImage(Mat8UC3_2QImage(getImg()));
    break;
  case CV_64FC1:
    m_QPixmap = QPixmap::fromImage(Mat64FC1_2QImage(getImg()));
    break;
  }
  setPixmap(m_QPixmap);

  repaint();
}

void QCVImage::Warp()
{
  int nMinLength = min(getImg().cols, getImg().rows);
  m_ptROIDst[0].x = m_ptROIDst[1].x = getImg().cols / 2 - nMinLength / 2;
  m_ptROIDst[2].x = m_ptROIDst[3].x = getImg().cols / 2 + nMinLength / 2;
  m_ptROIDst[0].y = m_ptROIDst[3].y = getImg().rows / 2 - nMinLength / 2;
  m_ptROIDst[1].y = m_ptROIDst[2].y = getImg().rows / 2 + nMinLength / 2;


  Point2f ptSrc[4], ptDst[4];
  for(int i=0; i<4; i++){
    ptSrc[i].x = (float) m_ptROISrc[i].x;
    ptSrc[i].y = (float) m_ptROISrc[i].y;
    ptDst[i].x = (float) m_ptROIDst[i].x;
    ptDst[i].y = (float) m_ptROIDst[i].y;
  }

  // warning
  // create(rows, cols, type) and cvCreateImage(cvSize(cols, rows), depth, channels)
  cv::Mat mxSrc = getImg();
  cv::Mat mxDst = cv::Mat(getImg().rows, getImg().cols, getImg().type());
  
  addImg(mxDst);

  CalibrationGeo cal;
  cal.Calibrate(ptSrc, ptDst, 4);
  cal.Transform(mxSrc, mxDst);
    
  for(int i=0; i<4; i++)
  {
    m_ptROISrc[i].x = m_ptROIDst[i].x;
    m_ptROISrc[i].y = m_ptROIDst[i].y;
  }

  update();
}

void QCVImage::ShadingCorrection(int nMeshSize)
{   
  vector<Mat> dst;
   
  int nSzX = m_ptROISrc[3].x-m_ptROISrc[0].x;
  int nSzY = m_ptROISrc[1].y-m_ptROISrc[0].y;
  int nMeshCntX = nSzX / nMeshSize;
  int nMeshCntY = nSzY / nMeshSize;
  int nMeshSzX = (nSzX / nMeshCntX);
  int nMeshSzY = (nSzY / nMeshCntY);
  nSzX = nMeshSzX*nMeshCntX;
  nSzY = nMeshSzY*nMeshCntY;
  Mat mxMeans(nMeshCntY, nMeshCntX, CV_8U);

  Mat mxLab_U8;
  Mat mxCurrent = getImg();
  if(true)
  {
    cvtColor(mxCurrent, mxLab_U8, CV_LBGR2Lab);//CV_BGR2Lab
  }
  else
  {  
    Mat mxRGB_F32;
    mxCurrent.convertTo(mxRGB_F32, CV_32F, 1./255.);
    Mat mxLab_F32;
    cvtColor(mxRGB_F32, mxLab_F32, CV_LBGR2Lab);
    mxLab_F32.convertTo(mxLab_U8, CV_8U, 255.);
  }

  getImg() = mxLab_U8;

  int nMeanLightness = 0;
  {
    for(int j=0; j<nMeshCntY; j++)
    {
      for(int i=0; i<nMeshCntX; i++)
      {
        int nSumLightness = 0;
        for(int _j=0; _j<nMeshSzY; _j++)
        {
          for(int _i=0;_i<nMeshSzX; _i++)
          {
            Vec3b intensity = mxLab_U8.at<Vec3b>(m_ptROISrc[0].y + j*nMeshSzY + _j, m_ptROISrc[0].x + i*nMeshSzX + _i);
            nSumLightness += intensity[0];
          }
        }
        mxMeans.at<uchar>(j, i) = nSumLightness / (nMeshSzX * nMeshSzY);
        nMeanLightness += mxMeans.at<uchar>(j, i);
      }
    }
    nMeanLightness /= nMeshCntX*nMeshCntY;
  }
  int i, j, _i, _j;
  int nMeanTL, nMeanTR, nMeanBL, nMeanBR, nLocalMean, nLightness;
  for(int r=0; r<nSzY; r++)
  {
    for(int c=0; c<nSzX; c++)
    {
      i = c / nMeshSzX;
      j = r / nMeshSzY;
      nMeanTL = (mxMeans.at<uchar>(max(0,j-1), max(0,i-1)) + mxMeans.at<uchar>(max(0,j-1), i)
               + mxMeans.at<uchar>(j, max(0,i-1)) + mxMeans.at<uchar>(j, i)) / 4;
      nMeanTR = (mxMeans.at<uchar>(max(0,j-1), i) + mxMeans.at<uchar>(max(0,j-1), min(nMeshCntX-1,i+1))
               + mxMeans.at<uchar>(j, i) + mxMeans.at<uchar>(j, min(nMeshCntX-1,i+1))) / 4;
      nMeanBL = (mxMeans.at<uchar>(j, max(0,i-1)) + mxMeans.at<uchar>(j, i)
               + mxMeans.at<uchar>(min(nMeshCntY-1,j+1), max(0,i-1)) + mxMeans.at<uchar>(min(nMeshCntY-1,j+1), i)) / 4;
      nMeanBR = (mxMeans.at<uchar>(j, i) + mxMeans.at<uchar>(j, min(nMeshCntX-1,i+1))
               + mxMeans.at<uchar>(min(nMeshCntY-1,j+1), i) + mxMeans.at<uchar>(min(nMeshCntY-1,j+1), min(nMeshCntX-1,i+1))) / 4;

      _i = c % nMeshSzX;
      _j = r % nMeshSzY;
      nLocalMean = (nMeanTL*(nMeshSzX-_i + nMeshSzY-_j) + nMeanTR*(_i + nMeshSzY-_j) + nMeanBL*(nMeshSzX-_i + _j) + nMeanBR*(_i+_j)) / (2*(nMeshSzX+nMeshSzY));
      nLightness = mxLab_U8.at<Vec3b>(m_ptROISrc[0].y+r, m_ptROISrc[0].x+c)[0];
      mxLab_U8.at<Vec3b>(m_ptROISrc[0].y+r, m_ptROISrc[0].x+c)[0] = (nMeanLightness * nLightness) / nLocalMean;
    }
  }
  
  cvtColor(mxLab_U8, getImg(), CV_Lab2LBGR);
  
  update();
}

inline bool isInsideRect(int nX, int nY, int nXUpL, int nYUpL, int nXDnR, int nYDnR)
{
  if(nX > nXUpL && nX < nXDnR && nY > nYUpL && nY < nYDnR)
    return true;
  else
    return false;
}


void QCVImage::mouseMouve(int nX, int nY, int nButton)
{  
  nX /= m_f64Zoom;
  nY /= m_f64Zoom;

  if(nButton && Qt::LeftButton)
  {
    if(m_nHandleStart < 4)
    {
      m_ptROISrc[m_nHandleStart].x = nX;
      m_ptROISrc[m_nHandleStart].y = nY;
      
      repaint();
    }
    else if(m_nHandleStart == 4)
    {
      for(int i=0; i<4; i++)
      {
        m_ptROISrc[i].x += nX - m_ptHandleStart.x;
        m_ptROISrc[i].y += nY - m_ptHandleStart.y;
      }
      m_ptHandleStart.x = nX;
      m_ptHandleStart.y = nY;

      repaint();
    }
    else
    {
      m_nHandleStart = m_nHandleMove;
      m_ptHandleStart.x = nX;
      m_ptHandleStart.y = nY;
    }
  }
  else
  {
    m_nHandleMove = m_nHandleStart = ROI_HANDLE_OUT;
    int nROIHandlerHalfSz = (ROI_HANDLER_HF/m_f64Zoom);
    for(int i=0; i<4; i++)
    {
      if(isInsideRect(nX, nY, m_ptROISrc[i].x - nROIHandlerHalfSz, m_ptROISrc[i].y - nROIHandlerHalfSz, m_ptROISrc[i].x + nROIHandlerHalfSz, m_ptROISrc[i].y + nROIHandlerHalfSz))
      {
        m_nHandleMove = i;
        break;
      }
    }
    if(m_nHandleMove == ROI_HANDLE_OUT)
    {
      if(isInsideRect(nX, nY, m_ptROISrc[0].x, m_ptROISrc[0].y, m_ptROISrc[2].x, m_ptROISrc[2].y))
        m_nHandleMove = ROI_HANDLE_IN;
    }

    if(m_nHandleMove == ROI_HANDLE_UPL || m_nHandleMove == ROI_HANDLE_DNR)
      setCursor(Qt::SizeFDiagCursor);
    else if(m_nHandleMove == ROI_HANDLE_UPR || m_nHandleMove == ROI_HANDLE_DNL)
      setCursor(Qt::SizeBDiagCursor);
    else if(m_nHandleMove == ROI_HANDLE_IN)
      setCursor(Qt::SizeAllCursor);
    else
      setCursor(Qt::ArrowCursor);
  }
}

void QCVImage::zoom(double f64Zoom)
{
  m_f64Zoom *= f64Zoom;
  resize(m_f64Zoom * pixmap()->size());
}
