/****************************************************************************
**
** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

//! [0]
#include <QtGui>
#if QT_VERSION > 0x050000
#include <QtWidgets>
#endif

#include "mainwindow.h"
#include "QImageScrollArea.h"

MainWindow::MainWindow()
{
  m_pDockWidget = NULL;
  m_pQScrollArea = new QImageScrollArea;
  setCentralWidget(m_pQScrollArea);

  createActions();
  createMenus();
  createToolBars();
  createStatusBar();
  createDockWindows();

  setWindowTitle(tr("Base Image Viewer"));
  resize(800, 800);
}

void MainWindow::about()
{
  QMessageBox::about(this, tr("About Base Image Viewer"),
    tr("The <b>Base Image Viewer</b> example demonstrates how to "
    "use QImageScrollArea and QPArametersDialog."));
}

void MainWindow::createActions()
{
  m_pNewAct = new QAction(QIcon(":/images/new.png"), tr("&New image"), this);
  m_pNewAct->setShortcuts(QKeySequence::New);
  m_pNewAct->setStatusTip(tr("Create a new empty image"));
  connect(m_pNewAct, SIGNAL(triggered()), m_pQScrollArea, SLOT(newSlot()));

  m_pOpenAct = new QAction(QIcon(":/images/open.png"), tr("&Open..."), this);
  m_pOpenAct->setShortcuts(QKeySequence::Open);
  m_pOpenAct->setStatusTip(tr("Open an image"));
  connect(m_pOpenAct, SIGNAL(triggered()), m_pQScrollArea, SLOT(openSlot()));

  m_pSaveAct = new QAction(QIcon(":/images/save.png"), tr("&Save..."), this);
  m_pSaveAct->setShortcuts(QKeySequence::Save);
  m_pSaveAct->setStatusTip(tr("Save the current image"));
  connect(m_pSaveAct, SIGNAL(triggered()), m_pQScrollArea, SLOT(saveSlot()));
  

  m_pWarpAct = new QAction(QIcon(":/images/roi.png"), tr("Warp..."), this);
  m_pWarpAct->setStatusTip(tr("Warp the current image"));
  connect(m_pWarpAct, SIGNAL(triggered()), m_pQScrollArea, SLOT(warp()));

  m_pShadingAct = new QAction(QIcon(":/images/shading.png"), tr("Shading Correction..."), this);
  m_pShadingAct->setStatusTip(tr("Correct spatial drift on the current image"));
  connect(m_pShadingAct, SIGNAL(triggered()), m_pQScrollArea, SLOT(shading()));

  /*
  m_pPrintAct = new QAction(QIcon(":/images/print.png"), tr("&Print..."), this);
  m_pPrintAct->setShortcuts(QKeySequence::Print);
  m_pPrintAct->setStatusTip(tr("Print the current image"));
  connect(m_pPrintAct, SIGNAL(triggered()), this, SLOT(print()));
  */

  m_pQuitAct = new QAction(tr("&Quit"), this);
  m_pQuitAct->setShortcuts(QKeySequence::Quit);
  m_pQuitAct->setStatusTip(tr("Quit the application"));
  connect(m_pQuitAct, SIGNAL(triggered()), this, SLOT(close()));

  m_pAboutAct = new QAction(tr("&About"), this);
  m_pAboutAct->setStatusTip(tr("Show the application's About box"));
  connect(m_pAboutAct, SIGNAL(triggered()), this, SLOT(about()));
}

void MainWindow::createMenus()
{
  m_pFileMenu = menuBar()->addMenu(tr("&File"));
  m_pFileMenu->addAction(m_pNewAct);
  m_pFileMenu->addAction(m_pOpenAct);
  m_pFileMenu->addAction(m_pSaveAct);
  //m_pFileMenu->addAction(m_pPrintAct);
  m_pFileMenu->addSeparator();
  m_pFileMenu->addAction(m_pQuitAct);

  m_pViewMenu = m_pQScrollArea->createMenus();//menuBar()->addMenu(tr("&View"));
  menuBar()->addMenu(m_pViewMenu);
  //m_pViewMenu->addMenu( m_pQScrollArea->createMenus());

  m_pHelpMenu = menuBar()->addMenu(tr("&Help"));
  m_pHelpMenu->addAction(m_pAboutAct);
}

void MainWindow::createToolBars()
{
  m_pFileToolBar = addToolBar(tr("File"));
  m_pFileToolBar->addAction(m_pNewAct);
  m_pFileToolBar->addAction(m_pOpenAct);
  m_pFileToolBar->addAction(m_pSaveAct);

  m_pCalibrationToolBar = addToolBar(tr("Calibration"));
  m_pCalibrationToolBar->addAction(m_pWarpAct);
  m_pCalibrationToolBar->addAction(m_pShadingAct);
  //m_pFileToolBar->addAction(m_pPrintAct);
}

void MainWindow::createStatusBar()
{
  //statusBar()->showMessage(tr("Ready"));
}

#include "QColorimeterDlg.h"

void MainWindow::createDockWindows()
{
  m_pDockWidget = new QDockWidget(tr("Parameters"), this);
  m_pDockWidget->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);

  QColorimeterDlg* pQColorimeterDlg = new QColorimeterDlg;
  pQColorimeterDlg->addQCVImage(m_pQScrollArea->getQCVImage());

  m_pDockWidget->setWidget(pQColorimeterDlg);
  addDockWidget(Qt::RightDockWidgetArea, m_pDockWidget);
  //m_pViewMenu->addAction(dock->toggleViewAction());
}
